package org.esjug.seamcms.application;

import java.util.List;

import javax.ejb.Remove;
import javax.ejb.Stateful;

import org.esjug.seamcms.domain.User;
import org.esjug.seamcms.persistence.UserDAO;
import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.AutoCreate;
import org.jboss.seam.annotations.Destroy;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.log.Log;

@AutoCreate
@Name("userCrud")
@Scope(ScopeType.SESSION)
@Stateful
public class UserCrudService implements UserCrud {
	@Logger
	private Log log;

	private List<User> users;

	@In
	private UserDAO userDAO;

	@Out(required = false)
	private User user;

	public List<User> getUsers() {
		if (users == null) {
			log.info("Retrieving all users from the DB...");
			users = userDAO.retrieveAll();
			log.info("#0 users retrieved.", users.size());
		}
		return users;
	}

	public void setUsers(List<User> users) {
		this.users = users;
	}

	public void create() {
		log.info("Creating a new user...");
		user = new User();
	}

	public void cancel() {
		log.info("Canceling user creation...");
		user = null;
	}

	public void save() {
		log.info("Saving user: #0 (#1)", user.getName(), user.getEmail());
		userDAO.save(user);
		users = null;
		user = null;
	}

	public void refresh() {
		log.info("Refreshing user list...");
		users = null;
	}

	@Remove
	@Destroy
	public void remove() {
	}
}
