package org.esjug.seamcms.domain;

import java.io.Serializable;
import java.util.UUID;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.MappedSuperclass;
import javax.persistence.Version;

@MappedSuperclass
public abstract class DomainObject implements Serializable {
	@Basic
	@Column(nullable = false, length = 40)
	protected String uuid;

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	private Long id;

	@Version
	@Column(nullable = false)
	private Long version;

	public DomainObject() {
		uuid = UUID.randomUUID().toString();
	}

	public String getUuid() {
		return uuid;
	}

	public void setUuid(String uuid) {
		this.uuid = uuid;
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public Long getVersion() {
		return version;
	}

	public void setVersion(Long version) {
		this.version = version;
	}

	@Override
	public boolean equals(Object obj) {
		// Checks if the class is the same.
		if ((obj == null) || (!getClass().equals(obj.getClass())))
			return false;

		// Checks if the UUID is the same.
		DomainObject o = (DomainObject) obj;
		return uuid.equals(o.uuid);
	}

	@Override
	public int hashCode() {
		return uuid.hashCode();
	}

	@Override
	public String toString() {
		return "Instance of " + getClass().getName() + " (id: " + id
				+ "; uuid: " + uuid + ")";
	}
}
