package org.esjug.seamcms.persistence;

import java.util.List;
import java.util.Map;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.esjug.seamcms.domain.DomainObject;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Transactional;
import org.jboss.seam.log.Log;

public abstract class SeamBaseDAO<T extends DomainObject> implements BaseDAO<T> {
	@Logger
	private Log log;

	// Concrete subclasses must provide entity manager.
	protected abstract EntityManager getEntityManager();

	// Base operations need to know the class.
	protected abstract Class<T> getDomainClass();

	// Optionally, subclasses can define order.
	protected String getOrderByClause() {
		return "";
	}

	@SuppressWarnings("unchecked")
	@Transactional
	public List<T> retrieveAll() {
		log.info("Retrieving all #0", getDomainClass().getName());
		Query query = getEntityManager().createQuery(
				"from " + getDomainClass().getName() + " obj "
						+ getOrderByClause());
		return query.getResultList();
	}

	@Transactional
	public T retrieveById(Long id) {
		log.info("Retrieving #0 with id #1", getDomainClass().getName(), id);
		return (T) getEntityManager().find(getDomainClass(), id);
	}

	@SuppressWarnings("unchecked")
	@Transactional
	public long retrieveCount() {
		Query query = getEntityManager().createQuery(
				"select new map (count(obj) as num) from "
						+ getDomainClass().getName() + " obj");
		Map<String, Object> data = (Map<String, Object>) query
				.getSingleResult();
		Long count = (Long) data.get("num");
		log.info("Retrieving count for #0: #1", getDomainClass().getName(),
				count);
		return (count == null) ? 0 : count;
	}

	@Transactional
	public void save(T object) {
		log.info("Saving #0: #1", getDomainClass().getName(), object);
		getEntityManager().persist(object);
	}

	@Transactional
	public void delete(T object) {
		log.info("Deleting #0: #1", getDomainClass().getName(), object);
		getEntityManager().remove(object);
	}
}
